%macro   salhsum;
options &mprint &source &notes;
%let dwdset=SALHSUM;

options metarepository="&HRRepos";
%LibConOp(sesetl_sasLibRef=&hcmlib,
         sesetl_ChkDepRepos=Y
         ) ;
proc sql;
    connect to &sesetl_EngineType as PASSTHRU
               (&sesetl_BestLogin &sesetl_ConnectionOptions );
       execute(drop table if exists salhsum
               ) by PASSTHRU;
    disconnect from PASSTHRU;
QUIT;

/* make sure work library has no current copies of the needed work datasets */
%deltable(tables=keys comphist compsum compsumt salhist salhist1 salhist2 salhist3 salhist4 salhist5);

%let basekeys=employee_id edate;
%let joinkeys=job_cd &hiers pay_level_structure_cd position_cd;
/* Summarize additional compensation data by id and time */
data comphist;
  length label $40 compensation_type_cd  $200;
  set &hcmlib..comphist(rename=(payment_dt=olddate));
  keep &basekeys compensation_type_cd compensation_amt label;

  /* keep only transactions occuring within the timeframe */
  if olddate ge "&slcutoff"d  and olddate le "&BLDCTOFF"D ;

  /* calculate the next cutoff anniversary date */
  edate=intnx("&salunit",olddate,0,"&salstart");
  if edate lt olddate then edate=intnx("&salunit",olddate,1,"&salstart");
  edate=min(edate,"&BLDCTOFF"D);
  label=put(compensation_type_cd,$comptyp.);
run;

/* summarize to one obs per item per period per person */
proc summary data=comphist nway;
  class compensation_type_cd &basekeys;
  var compensation_amt;
  id label;
  output out=compsum(drop=_type_ _freq_) sum=;
run;

/* process the summary output to make the comptype values unique */
/* in the first 8 characters.  This is so the transpose can use  */
/* the formated value of COMPTYPE as the label for the new variable */
/* Also, create macro string to be used on in a SUM function in a */
/* later data step */
%let sumstrng=;
%global addn;
%let addn=0;
%let fmtstrng=;

data compsum;
  set compsum end=end;
  by compensation_type_cd;
  if first.compensation_type_cd then do;
   num+1;
   Call symput('add'||strip(put(num,5.)),strip(label));  /*Write the Compensation Code label to a macro variable.*/
  End;
  compensation_type_cd='ADD'!!left(put(num,5.));
  if end then do;
    call symput('addn',left(put(num,5.)));
    if num=1 then do;
      call symput('SUMSTRNG',', ADD1');
      call symput('FMTSTRNG',"format ADD1 &dllrfrmt;");
    end;
    else do;
      call symput('SUMSTRNG',', of ADD1-ADD'!!left(put(num,5.)));
      call symput('FMTSTRNG',"format ADD1-ADD"!!left(put(num,5.))!!" &dllrfrmt;");
    end;
  end;
  drop num;
run;

/*Create Global macro variables for the labels to be used by the %updatdd macro.*/
%if &addn.>0 %then %do;
 %Do i=1 %to &addn.;
  %global add&i._label;
  %let add&i._label=%superq(add&i.);
 %End;
%End;

/* transpose the compensation types to individual variables */
/* use the previously generated labels as the new variable names */
proc sort data=compsum out=compsum;
  by &basekeys compensation_type_cd;
run;
proc transpose data=compsum
               out=compsumt(drop=_name_ _label_);
  by &basekeys;
  var compensation_amt;
  id compensation_type_cd;
  idlabel label;
run;
proc datasets nolist lib=work;
  modify compsumt;
  &fmtstrng
run;
quit;

/* see if EMPSTAT is in ACTHIST, if so, generate a where clause */
/* to eliminate inactive employees */
%let empwhr=;
data _null_;
  dsid=open("&hcmlib..acthist");
  if dsid gt 0 then do;
    if varnum(dsid,'EMPLOYEE_STATUS_CD') gt 0 then
      call symput('EMPWHR',' and put(employee_status_cd,$iempsta.) ne "IN"');
  end;
run;

/* Generate one obs per full time unit for each person */
data salhist;
  set &hcmlib..acthist(drop=employee_action_reason_cd  _lastrec
                      rename=(action_dt=edate VALID_TO_DT=eedate)
                      where=(edate ne . and put(action_type_cd,$iaction.)
                      not in ('ITERM' 'VTERM') &empwhr));

  /* take care of the problem that the dates might not be integers */
  if edate ne . then edate=int(edate);
  if eedate ne . then eedate=int(eedate);

  /* create a working eedate, keep only records ending after the cutoff */
  if eedate=. then w_eedate="&bldctoff"d;
  else w_eedate=eedate;
  drop w_eedate;

  /* get the anniversay just after edate */
  edatev=intnx("&salunit",edate,0,"&salstart");
  if edatev lt edate then edatev=intnx("&salunit",edate,1,"&salstart");
  drop edatev;

  /* get the anniversay just before eedate */
  eedatev=intnx("&salunit",w_eedate,0,"&salstart");
  if eedatev gt w_eedate then eedatev=intnx("&salunit",w_eedate,-1,"&salstart");
  drop eedatev;

  /* if the record is the current record for the employee, generate a current record */
  if w_eedate="&bldctoff"d and edatev ge "&slcutoff"d then do;
    edate="&bldctoff"d;
    output;
  end;

  /* keep only records that are not on the same time unit */
  if eedatev ge edatev;

  /* keep only records valid after the cutoff and before the build date */
  if eedatev ge "&slcutoff"d and edatev le "&bldctoff"d;

  /* reset edatev to the next anniversary after/equal to the cutoff */
  edatev=max(edatev,"&slcutoff"d);

  /* reset eedatev to the anniversary before/equal to the bldcutoff date */
  eedatev=min(eedatev,"&bldctoff"d);

  /* generate observations for each time unit in the time span */
  nunits=intck("&salunit",edatev,eedatev);
  if nunits ge 0 then do i=0 to nunits;
    edate=intnx("&salunit",edatev,i,"&salstart");
    output;
  end;
  drop i nunits eedate action_type_cd;

  format edate &datefrmt;
run;

/* make sure SALHIST is in the proper order */
proc sort data=salhist out=salhist;
  by &basekeys;
run;

/* remove any duplicated obs for a given id and time unit */
/* calculate salary changes */
data salhist;
  set salhist;
  by &basekeys;
  if last.edate;
run;


/* build the initial keys data set */
%joinkeys(basetabl=salhist,basekeys=&basekeys,joinkeys=&joinkeys,init=Y);

/* join in position information */
%hrjoin(table=salhist1,base=keys,basejvar=position_cd,
        join=&hcmlib..pos,joinjvar=position_cd,
        basekey=&basekeys,basedate=edate,joindate=VALID_FROM_DT,
        dropvars=&posm job_cd employee_id _lastrec,eedtevar=VALID_TO_DT);


/* see if we need to pick up additional keys from pos */
%joinkeys(basetabl=salhist1,basekeys=&basekeys,joinkeys=&joinkeys);

/* join in job information */
%hrjoin(table=salhist2,base=keys,basejvar=job_cd,
        join=&hcmlib..jobs,joinjvar=job_cd,
        basekey=&basekeys,basedate=edate,joindate=VALID_FROM_DT,
        dropvars=&jobsm  _lastrec,eedtevar=VALID_TO_DT);


/* see if we need to pick up additional keys from jobs */
%joinkeys(basetabl=salhist2,basekeys=&basekeys,joinkeys=&joinkeys);

/* join in hierarchy information */
data keyshr(drop=pay_level_structure_cd position_cd job_cd);
      set keys;run;
%grphiers(baseds=keyshr,grplevds=&hcmlib..wrkgrp,basedate=VALID_FROM_DT,eedtevar=VALID_TO_DT,
         hiername=hiermax,rptgrp=intorg,prntgrp=parent_intorg,
         outds=rptgrphr,joindate=VALID_FROM_DT,mgrname=MANAGER_NAME);
data salhist3;
 set rptgrphr(drop=grpdesc hierarchy_cd);
run;

/* join in pay grade information */
%hrjoin(table=salhist4,base=keys,basejvar=pay_level_structure_cd,
        join=&hcmlib..grade,joinjvar=pay_level_structure_cd,
        basekey=&basekeys,basedate=edate,joindate=VALID_FROM_DT,
        dropvars=&gradem  _lastrec,eedtevar=VALID_TO_DT);


/* see if we need to pick up additional keys from grade */
%joinkeys(basetabl=salhist4,basekeys=&basekeys,joinkeys=&joinkeys);

/* join in employee general information */
%cempgen;
%hrjoin(table=salhist5,base=keys,basejvar=employee_id,
        join=cempgen,joinjvar=employee_id,
        basekey=&basekeys,dropvars=&empgenm  VALID_FROM_DT VALID_TO_DT _lastrec
                                    source_system_cd,eedtevar=VALID_TO_DT);


/* add calculated fields */
/* add in additional comp buckets to salhist to get totcomp */

data &hcmlib..SALHSUM(rename=(edate=VALID_DT));
  merge
        salhist5
        salhist4
        salhist3
        salhist2
        salhist1
        compsumt
        salhist(in=s)
  ;
  by &basekeys;
  if s;

  TOTOTHER=sum(0 &sumstrng);
  TOTCOMP=sum(annual_salary &sumstrng);

  lagsal=lag(annual_salary);
  if first.employee_id then lagsal=.;
  if last.edate;

  if not first.employee_id then do;
    if lagsal ne . then
      CHGAMT=annual_salary-lagsal;
    if lagsal not in (.,0) then
      CHGPCT=chgamt/lagsal;
  end;

  if last.employee_id then _LASTREC='Y';else _LASTREC='N';

  /* use only active employees */
  if put(employee_status_cd,$iempsta.) ne "IN";

  /* recalculate years of service based on service date and cutoff date */
  %LOS(basedate=edate)

  /* recalculate age based on cutoff date */
  %AGE(basedate=edate)

  drop lagsal;

  /* drop any additional variables */
  %DROPVAR(dropvar=salhsumv);

  format edate date9.;

run;


/* clean up */
%deltable(tables=keys comphist compsum compsumt salhist salhist1 salhist2 salhist3 salhist4 salhist5
          rptgrphr keyshr);

%regrdata(sesetl_altETLStudioTreeNm=SAS Data Integration Studio Custom Tree,
          sesetl_altEtlStudioGroupNm=Data Integration Studio Groups,
          sesetl_sasLibrary=&hcmlib,sesetl_physicalTable=SALHSUM);

%mend;
